import { diffLines } from 'diff';
import Prism$1 from 'prismjs';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

(function(Prism) {

var javascript = Prism.util.clone(Prism.languages.javascript);

Prism.languages.jsx = Prism.languages.extend('markup', javascript);
Prism.languages.jsx.tag.pattern= /<\/?(?:[\w.:-]+\s*(?:\s+(?:[\w.:$-]+(?:=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s{'">=]+|\{(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}))?|\{\s*\.{3}\s*[a-z_$][\w$]*(?:\.[a-z_$][\w$]*)*\s*\}))*\s*\/?)?>/i;

Prism.languages.jsx.tag.inside['tag'].pattern = /^<\/?[^\s>\/]*/i;
Prism.languages.jsx.tag.inside['attr-value'].pattern = /=(?!\{)(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">]+)/i;
Prism.languages.jsx.tag.inside['tag'].inside['class-name'] = /^[A-Z]\w*(?:\.[A-Z]\w*)*$/;

Prism.languages.insertBefore('inside', 'attr-name', {
	'spread': {
		pattern: /\{\s*\.{3}\s*[a-z_$][\w$]*(?:\.[a-z_$][\w$]*)*\s*\}/,
		inside: {
			'punctuation': /\.{3}|[{}.]/,
			'attr-value': /\w+/
		}
	}
}, Prism.languages.jsx.tag);

Prism.languages.insertBefore('inside', 'attr-value',{
	'script': {
		// Allow for two levels of nesting
		pattern: /=(?:\{(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\})/i,
		inside: {
			'script-punctuation': {
				pattern: /^=(?={)/,
				alias: 'punctuation'
			},
			rest: Prism.languages.jsx
		},
		'alias': 'language-javascript'
	}
}, Prism.languages.jsx.tag);

// The following will handle plain text inside tags
var stringifyToken = function (token) {
	if (!token) {
		return '';
	}
	if (typeof token === 'string') {
		return token;
	}
	if (typeof token.content === 'string') {
		return token.content;
	}
	return token.content.map(stringifyToken).join('');
};

var walkTokens = function (tokens) {
	var openedTags = [];
	for (var i = 0; i < tokens.length; i++) {
		var token = tokens[i];
		var notTagNorBrace = false;

		if (typeof token !== 'string') {
			if (token.type === 'tag' && token.content[0] && token.content[0].type === 'tag') {
				// We found a tag, now find its kind

				if (token.content[0].content[0].content === '</') {
					// Closing tag
					if (openedTags.length > 0 && openedTags[openedTags.length - 1].tagName === stringifyToken(token.content[0].content[1])) {
						// Pop matching opening tag
						openedTags.pop();
					}
				} else {
					if (token.content[token.content.length - 1].content === '/>') ; else {
						// Opening tag
						openedTags.push({
							tagName: stringifyToken(token.content[0].content[1]),
							openedBraces: 0
						});
					}
				}
			} else if (openedTags.length > 0 && token.type === 'punctuation' && token.content === '{') {

				// Here we might have entered a JSX context inside a tag
				openedTags[openedTags.length - 1].openedBraces++;

			} else if (openedTags.length > 0 && openedTags[openedTags.length - 1].openedBraces > 0 && token.type === 'punctuation' && token.content === '}') {

				// Here we might have left a JSX context inside a tag
				openedTags[openedTags.length - 1].openedBraces--;

			} else {
				notTagNorBrace = true;
			}
		}
		if (notTagNorBrace || typeof token === 'string') {
			if (openedTags.length > 0 && openedTags[openedTags.length - 1].openedBraces === 0) {
				// Here we are inside a tag, and not inside a JSX context.
				// That's plain text: drop any tokens matched.
				var plainText = stringifyToken(token);

				// And merge text with adjacent text
				if (i < tokens.length - 1 && (typeof tokens[i + 1] === 'string' || tokens[i + 1].type === 'plain-text')) {
					plainText += stringifyToken(tokens[i + 1]);
					tokens.splice(i + 1, 1);
				}
				if (i > 0 && (typeof tokens[i - 1] === 'string' || tokens[i - 1].type === 'plain-text')) {
					plainText = stringifyToken(tokens[i - 1]) + plainText;
					tokens.splice(i - 1, 1);
					i--;
				}

				tokens[i] = new Prism.Token('plain-text', plainText, null, plainText);
			}
		}

		if (token.content && typeof token.content !== 'string') {
			walkTokens(token.content);
		}
	}
};

Prism.hooks.add('after-tokenize', function (env) {
	if (env.language !== 'jsx' && env.language !== 'tsx') {
		return;
	}
	walkTokens(env.tokens);
});

}(Prism));

var newlineRe = /\r\n|\r|\n/;
Prism$1.manual = true;
function tokenize(code, lang) {
    var grammar = Prism$1.languages[lang];
    if (!grammar) {
        throw new MissingGrammarError(lang);
    }
    var prismTokens = Prism$1.tokenize(code, Prism$1.languages[lang]);
    var nestedTokens = tokenizeStrings(prismTokens);
    var tokens = flattenTokens(nestedTokens);
    var currentLine = [];
    var lines = [currentLine];
    tokens.forEach(function (token) {
        var contentLines = token.content.split(newlineRe);
        var firstContent = contentLines.shift();
        if (firstContent !== undefined && firstContent !== "") {
            currentLine.push([firstContent, token.type]);
        }
        contentLines.forEach(function (content) {
            currentLine = [];
            lines.push(currentLine);
            if (content !== "") {
                currentLine.push([content, token.type]);
            }
        });
    });
    return lines;
}
function tokenizeStrings(prismTokens, parentType) {
    if (parentType === void 0) { parentType = "plain"; }
    return prismTokens.map(function (prismToken) {
        return wrapToken(prismToken, parentType);
    });
}
function wrapToken(prismToken, parentType) {
    if (parentType === void 0) { parentType = "plain"; }
    if (typeof prismToken === "string") {
        return {
            type: parentType,
            content: prismToken,
        };
    }
    if (Array.isArray(prismToken.content)) {
        return {
            type: prismToken.type,
            content: tokenizeStrings(prismToken.content, prismToken.type),
        };
    }
    return wrapToken(prismToken.content, prismToken.type);
}
// Take a list of nested tokens
// (token.content may contain an array of tokens)
// and flatten it so content is always a string
// and type the type of the leaf
function flattenTokens(tokens) {
    var flatList = [];
    tokens.forEach(function (token) {
        var type = token.type, content = token.content;
        if (Array.isArray(content)) {
            flatList.push.apply(flatList, __spread(flattenTokens(content)));
        }
        else {
            flatList.push({ type: type, content: content });
        }
    });
    return flatList;
}
var MissingGrammarError = /** @class */ (function (_super) {
    __extends(MissingGrammarError, _super);
    function MissingGrammarError(lang) {
        var _newTarget = this.constructor;
        var _this = _super.call(this, "Missing syntax highlighting for language \"" + lang + "\"") || this;
        _this.lang = lang;
        Object.setPrototypeOf(_this, _newTarget.prototype);
        return _this;
    }
    return MissingGrammarError;
}(Error));

function codeDiff(_a) {
    var prevCode = _a.prevCode, nextCode = _a.nextCode, lineKeys = _a.lineKeys, lang = _a.lang;
    var prevNormalCode = normalize(prevCode || "");
    var nextNormalCode = normalize(nextCode || "");
    var prevLines = tokenize(prevNormalCode, lang);
    var nextLines = tokenize(nextNormalCode, lang);
    var prevKeys = lineKeys ||
        Array.from({ length: prevLines.length - 1 }, function (x, i) { return i + 1; });
    var nextKeys = getNextKeys(prevNormalCode, nextNormalCode, prevKeys);
    var codeMap = {};
    prevKeys.forEach(function (key, index) { return (codeMap[key] = prevLines[index]); });
    nextKeys.forEach(function (key, index) { return (codeMap[key] = nextLines[index]); });
    return { prevKeys: prevKeys, nextKeys: nextKeys, codeMap: codeMap };
}
function getNextKeys(prevCode, nextCode, prevKeys) {
    var changes = diffLines(prevCode, nextCode);
    var nextKeys = [];
    var prevIndex = 0;
    changes.forEach(function (change) {
        if (change.added) {
            var p = prevKeys[prevIndex - 1] != null
                ? prevKeys[prevIndex - 1]
                : prevKeys[prevIndex] - 1;
            var n = prevKeys[prevIndex] != null
                ? prevKeys[prevIndex]
                : prevKeys[prevKeys.length - 1] + 1;
            for (var i = 1; i <= change.count; i++) {
                nextKeys.push(p + ((n - p) * i) / (change.count + 1));
            }
        }
        else if (change.removed) {
            prevIndex += change.count;
        }
        else {
            for (var i = 1; i <= change.count; i++) {
                nextKeys.push(prevKeys[prevIndex]);
                prevIndex++;
            }
        }
    });
    return nextKeys;
}
function normalize(text) {
    return text && text.trimEnd().concat("\n");
}

export { codeDiff };
//# sourceMappingURL=index.esm.js.map
